import {browser, ElementFinder, promise, protractor} from 'protractor';
import {ElementLocators} from "./ElementLocators";

export class UIOperations extends ElementLocators {
    EC = protractor.ExpectedConditions;
    private TIMEOUT = 30000;


    constructor() {
        super();
        console.log("Super class constructor");
    }

    //Generic method for clicking an element.
    click(element: ElementFinder) {
        return this.waitForVisible(element).then(() => {
            this.waitForClickAble(element).then(() => {
                console.log('clicking on' + element.locator() + ']');
                element.click().then(() => {
                    console.log('clicked on' + element.locator() + ']');
                    return true;
                })
            }).catch((error) => {
                console.log('not clicked on' + element.locator() + ']');
                throw error;
            })
        })
    }

    waitForVisible(element: ElementFinder) {
        return browser
            .wait(
                this.EC.visibilityOf(element),
                this.TIMEOUT
            )
            .then(() => {
                console.log("element visible " + element.locator());
                return true;
            })
            .catch((error) => {
                console.log("element not visible " + element.locator());
                throw error;
            })
    }

    waitForClickAble(element: ElementFinder): promise.Promise<void> {
        return browser
            .wait(
                this.EC.elementToBeClickable(element),
                this.TIMEOUT
            )
            .then(() => {
                console.log("element clickable " + element.locator());
            })
            .catch((error) => {
               console.log("element not clickable " + element.locator());
                throw error;
            })
    }

    waitForInvisible(element: ElementFinder): promise.Promise<void> {
        return browser
            .wait(
                this.EC.invisibilityOf(element),
                this.TIMEOUT
            )
            .then(() => {
                console.log("element invisible " + element.locator());
            })
            .catch((error) => {
                console.log("element visible " + element.locator());
                throw error;
            })
    }

    waitForElementToBeSelected(element: ElementFinder) {
        return browser
            .wait(
                this.EC.invisibilityOf(element),
                this.TIMEOUT
            )
            .then(() => {
                console.log("element selected " + element.locator());
            })
            .catch((error) => {
                console.log("element not selected " + element.locator());
                throw error;
            })
    }

    waitForElementToBePresent(element: ElementFinder) {
        return browser
            .wait(
                protractor.ExpectedConditions.presenceOf(element),
                this.TIMEOUT
            )
            .then(() => {
                console.log("element present " + element.locator());
            })
            .catch((error) => {
                console.log("element not present " + element.locator());
                throw error;
            })
    }

    waitForElementToBeStale(element: ElementFinder) {
        return browser
            .wait(
                protractor.ExpectedConditions.stalenessOf(element),
                this.TIMEOUT
            )
            .then(() => {
                console.log("element not present " + element.locator());
            })
            .catch((error) => {
                console.log("element present " + element.locator());
                throw error;
            })
    }


    isDisplayed(element: ElementFinder): promise.Promise<boolean> {
        return element.isDisplayed();
    }

    isEnabled(element: ElementFinder): promise.Promise<boolean> {
        return element.isEnabled();
    }

    isPresent(element: ElementFinder): promise.Promise<boolean> {
        return element.isPresent()
    }

    isElementSelected(element: ElementFinder): promise.Promise<boolean> {
        return element.isSelected();
    }

    //Generic method for sending text into an element.
    sendKeys(element: ElementFinder, text: string): promise.Promise<void> {
        return this.clear(element).then(() => {
            element.sendKeys(text);
        })
    }

    //Generic method for clearing text of an element.
    clear(element: ElementFinder): promise.Promise<void> {
        return this.waitForVisible(element).then(() => {
            return element.clear();
        })
    }


    //Generic method for getting the specified attribute value of an element.
    getAttribute(element: ElementFinder, attribute: string): promise.Promise<string> {
        return this.waitForVisible(element).then(() => {
            return element.getAttribute(attribute);
        })
    }

    getTextToVerify(element: ElementFinder): promise.Promise<string> {
        return this.waitForVisible(element).then(() => {
            return element.getText();
        })
    }

    getTagName(element: ElementFinder): promise.Promise<string> {
        return this.waitForVisible(element).then(() => {
            return element.getTagName();
        })
    }

}